  # A class to provide the basic functionality to create a PDF document
  # without any requirement for additional modules.
  #
  # Based on the PHP PDF library[http://www.ros.co.nz/pdf] version 009 by
  # Wayne Munro (<pdf@ros.co.nz>).
  #
  # == About
  # Author::    Austin Ziegler
  # Copyright:: Copyright (c) 2003 Austin Ziegler
  # Licence::   MIT-Style
  #
  # == Licence
  #
  # Permission is hereby granted, free of charge, to any person obtaining a
  # copy of this software and associated documentation files (the "Software"),
  # to deal in the Software without restriction, including without limitation
  # the rights to use, copy, modify, merge, publish, distribute, sublicense,
  # and/or sell copies of the Software, and to permit persons to whom the
  # Software is furnished to do so, subject to the following conditions:
  #
  # The above copyright notice and this permission notice shall be included in
  # all copies or substantial portions of the Software.
  #
  # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
  # THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  # DEALINGS IN THE SOFTWARE.
  #
module PDF
  VERSION = '0.1.1'

  begin
    require 'zlib'
    Compression = true
  rescue
    Compression = false
  end

  Message = {}

  class << self
      # PDF is fully internationalized. This module method sets the error
      # messages to the specified language Module. The language Module must
      # have a constant Hash called +Message+ containing a set of symbols and
      # localized versions of the messages associated with them.
      #
      # If the file 'pdf/lang_es' contains the module <tt>PDF::Espanol</tt>, the
      # error messages for PDF could be localized to Espa�ol thus:
      #
      #   require 'pdf/writer'
      #   require 'pdf/lang_es'
      #
      #   PDF.language = PDF::Espanol
      #
      # Localization is module-global; in a multithreaded program, all threads
      # will use the current language's messages.
      #
      # See PDF::English for more information.
    attr_accessor :language

    def language=(l) #:nodoc:
      @language = l
      Message.replace(l::Message)
    end
  end

    # PDF::English is the English-language output module. It contains a hash,
    # *Message*, that contains the messages that may be reported by any
    # method in the PDF library. The messages are identified by a Symbol.
    #
    # <b>Symbol</b>::         <b>Meaning</b>
    # <b>:uri_09x</b>::       URIs must be HTTP or FTP URIs in this version of RSS.
    #
  module English
    Message = {
      :invalid_pos          => ":pos must be either :before or :after.",
      :req_FPXO             => "Pages#<< and requires a PDF::Writer::Page, PDF::Writer::Font, or PDF::Writer::ExternalObject.",
      :req_FPXOH            => "Pages#add requires a PDF::Writer::Page, PDF::Writer::Font, PDF::Writer::ExternalObject, or Hash.",
      :png_invalid_header   => "Invalid PNG header.",
      :png_unsupp_compres   => "Unsupported PNG compression method.",
      :png_unsupp_filter    => "Unsupported PNG filter method.",
      :png_header_missing   => "PNG information header is missing.",
      :png_8bit_colour      => "Only PNG colour depths of 8 bits or less are supported.",
      :png_alpha_trans      => "PNG alpha channel transparency is not supported; only palette transparency is supported.",
      :data_must_be_array   => "The table data is not an Array. (Temporary limitation.)",
      :columns_unspecified  => "Columns are unspecified. They must be data[0] and be an array.",
    }
  end

  PDF.language = PDF::English

  class << self
      # Escape the text so that it's safe for insertion into the PDF document.
    def escape(text)
      text.gsub(/\\/, '\\\\\\\\').
          gsub(/\(/, '\\(').
          gsub(/\)/, '\\)').
          gsub(/&lt;/, '<').
          gsub(/&gt;/, '>').
          gsub(/&#039;/, "\\\\'").
          gsub(/&quot;/, '"').
          gsub(/&amp;/, '&')
    end

    def deg2rad(d)
      (d / 180.0) * Math::PI
    end

    def rad2deg(r)
      (r / Math::PI) * 180.0
    end
  end
end
