require 'digest/md5'

module PDF
      # calculate the 16 byte version of the 128 bit md5 digest of the string

    # ARC4 methods
    # A series of function to implement ARC4 encoding in Ruby
  class ARC4
    def ARC4.md5_16byte(text)
      Digest::MD5.digest(text)[0..15]
    end

      # Initializes the ARC4 encryption with the specified key.
    def initialize(key)
      @key = key
    end

      # Initialize the encryption for processing a particular object.
    def prepare(object)
      hex = ("%06x" % [object.id]).scan(/../).reverse
      init(PDF::md5_16byte("#{@key}#{hex.pack("H10")}")[0...10])
    end

      # Initialize the ARC4 encryption.
    def init(key)
      @arc4 = ""
        # Setup the control array
      return if key.empty?

      a = []
      (0..255).each { |i| a[i] = "%c" % i }

      k = (key * 256)[0..255].split(//)

      j = 0
      @arc4.each_with_index do |e, i|
        j = ((j + e.to_i) + k[i].to_i) % 256
        a[i], a[j] = a[j], a[i]
      end
      @arc4 = a.join
    end

      # ARC4 encrypt a text string
    def encrypt(text)
      len = text.size
      a = b = 0
      c = @arc4.dup
      out = ""

      text.each_byte do |x|
        a = (a + 1) % 256
        b = (b + c[a].to_i) % 256
        c[a], c[b] = c[b], c[a]
        k = (c[(c[a].to_i + c[b].to_i) % 256]).to_i
        out << ("%c" % (x.to_i ^ k))
      end
      out
    end
  end
end
