module PDF
  FONT_PATH = []

  class Font
    KEYS = %w{FontName FullName FamilyName Weight ItalicAngle IsFixedPitch
              CharacterSet UnderlinePosition UnderlineThickness Version
              EncodingScheme CapHeight XHeight Ascender Descender StdHW StdVW
              StartCharMetrics FontBBox C KPX}

    def initialize
      @kpx = {}
      @c = {}
      @version = 1
      @font_num = nil
    end

    KEYS.each { |k| attr_accessor k.downcase.intern }

    attr_accessor :font_num
    attr_accessor :path
    attr_accessor :differences

    NUMBER = /^[+\-0-9.]+$/

      # Open the font file and return a PDF::Font object containing it. The
      # +fontname+ may specify just a font file or a full path. If a path is
      # specified, that is the only place where the font file will be looked
      # for. 
    def Font.open(fontname)
      file  = fontname.gsub(/\\/, "/")
      dir   = [File.dirname(file)]
      name  = File.basename(file)

        # Check to see if the directory is "." or a non-path
      if dir[0] == "."
        dir << FONT_PATH
        dir << $LOAD_PATH
      elsif dir[0] !~ /^\w:/ and dir[0].index("/")
        FONT_PATH.each { |p| dir << File.join(p, dir[0]) }
        $LOAD_PATH.each { |p| dir << File.join(p, dir[0]) }
        dir[0] = "./#{dir[0]}"
      end

      font = nil

      afm = nil
      dir.each do |d|
        afm = File.join(d, "#{name}.afm").gsub(/\.afm\.afm$/, ".afm")
        rfm = "#{afm}.rfm"

          # Attempt to unmarshal an .afm.rfm file first.
        font = Marshal.load(rfm) rescue nil
          # Return the loaded font if it has been found.
        return font unless font.nil?

          # Attempt to open and process the font.
        File.open(afm) do |f|
          font = PDF::Font.new
          f.each do |l|
            key, *rest = l.chomp.strip.split
            case key
            when 'FontName', 'FullName', 'FamilyName', 'Weight',
                 'IsFixedPitch', 'CharacterSet', 'Version', 'EncodingScheme'
              font.__send__("#{key.downcase}=".intern, rest.join(" "))
            when 'ItalicAngle', 'UnderlinePosition', 'UnderlineThickness',
                 'CapHeight', 'XHeight', 'Ascender', 'Descender', 'StdHW',
                 'StdVW', 'StartCharMetrics'
              font.__send__("#{key.downcase}=".intern, rest.join(" ").to_f)
            when 'FontBBox'
              font.fontbbox = rest
            when 'C' # C 39 ; WX 222 ; N quoteright ; B 53 463 157 718 ;
              bits = l.chomp.strip.split(/;/).collect { |r| r.strip }
              dtmp = {}

              bits.each do |bit|
                b = bit.split
                if b.size > 2
                  dtmp[b[0]] = []
                  b[1..-1].each do |z|
                    if z =~ NUMBER
                      dtmp[b[0]] << z.to_f
                    else
                      dtmp[b[0]] << z
                    end
                  end
                elsif b.size == 2
                  if b[0] == 'C' and b[1] =~ NUMBER
                    dtmp[b[0]] = b[1].to_i
                  elsif b[1] =~ NUMBER
                    dtmp[b[0]] = b[1].to_f
                  else
                    dtmp[b[0]] = b[1]
                  end
                end
              end

              font.c[dtmp['N']] = dtmp
              font.c[dtmp['C']] = dtmp unless dtmp['C'].nil?
            when 'KPX' # KPX Adieresis yacute -40
              font.kpx[rest[0]] = { }
              font.kpx[rest[0]][rest[1]] = rest[2].to_f
            end
          end
          font.path = "#{d}"
        end rescue nil # Ignore file errors
        break unless font.nil?
      end

      raise TypeError, "Font #{fontname} not found." if font.nil?
      font.path = afm
      font
    end
  end
end
