module PDF
  class Writer
      # An annotation object, this will add an annotation to the current page.
      # initially will support just link annotations.
    class Annotation < PDF::Writer::WriterObject
      TYPES = [:link, :ilink]

      def initialize(parent, type, rect, label)
        super(parent)

        @type = type
        @rect = rect

        case @type
        when :link
          @action = PDF::Writer::Action.new(parent, label)
        when :ilink
          @action = PDF::Writer::Action.new(parent, label, type)
        end
        parent.current_page.add_annotation(self)
      end

      attr_accessor :type
      attr_accessor :action
      attr_accessor :rect

      def to_s
        res = "\n#{@id} 0 obj\n<< /Type /Annot"
        res << "\n/Subtype /Link" if TYPES.include?(@type)
        res << "\n/A #{@action.id} 0 R\n/Border [0 0 0]\n/H /I\n/Rect ["
        @rect.each { |v| res << "%.4f " % v }
        res << "]\n>>\nendobj"
      end
    end
  end
end
