module PDF
  class Writer
      # The contents objects hold all of the content which appears on pages
    class Contents < PDF::Writer::WriterObject
      def initialize(parent, page = nil)
        super(parent)

        @data = ""
        @info = {}
        @raw = false
        @on_page = nil

        if page.kind_of?(PDF::Writer::Page)
          @on_page = page
        elsif page == :raw
          @raw = true
        end
      end

      attr_reader :on_page

      def size
        @data.size
      end

      def each
        @contents.each { |c| yield c }
      end

      def <<(v)
        raise TypeError unless v.kind_of?(PDF::Writer::WriterObject) or v.kind_of?(String)
        @data << v
      end

      def add(a)
        a.each { |k, v| @info[k] = v }
      end

      def to_s
        tmp = @data.dup
        res = "\n#{@id} 0 obj\n"
        if @raw
          res << tmp
        else
          res << "<<"
          if PDF::Compression and @parent.options[:compression]
            res << " /Filter /FlateDecode"
            tmp = Zlib::Deflate(tmp)
          end
          if (@parent.encrypted?)
            @parent.arc4.prepare(self)
            tmp = @parent.arc4.encrypt(tmp)
          end
          @info.each { |k, v| res << "\n/#{k} #{v}" }
          res << "\n/Length #{tmp.size} >>\nstream\n#{tmp}\nendstream"
        end
        res << "\nendobj\n"
        res
      end
    end
  end
end
