module PDF
  class Writer
    # an object to hold the font description
    class Font < PDF::Writer::WriterObject
      Details = %w{FirstChar LastChar Widths FontDescriptor SubType}

      def initialize(parent, name, encoding = 'WinAnsiEncoding', subtype = 'Type1')
        super(parent)

        @name     = name
        @subtype  = subtype
        @encoding = encoding

        @font_id  = @parent.generate_font_id

        if encoding.kind_of?(FontEncoding)
          @encoding = nil
          @encodingDictionary = encoding
        elsif encoding == 'none' or encoding.nil?
          @encoding = nil
        end

        @parent.pages << self
      end

      attr_reader :font_id
      attr_accessor :subtype
        # Valid values: WinAnsiEncoding, MacRomanEncoding, MacExpertEncoding,
        # none, +nil+
      attr_accessor :encoding
      attr_reader :encodingDictionary
      def encodingDictionary=(ed)
        @encodingDictionary = ed
      end

      def basefont
        @name
      end

      def basefont=(b)
        @name = basefont
      end

      Details.each do |d|
        attr_accessor d.downcase.intern
      end

      def to_s
        res = "\n#{@id} 0 obj\n<< /Type /Font\n/Subtype /#{@subtype}\n"
        res << "/Name /F#{@font_id}\n/BaseFont /#{@name}\n"
        if @encodingDictionary.nil?
          res << "/Encoding /#{@encoding}\n"
        else
          res << "/Encoding #{@encodingDictionary.id} 0 R\n"
        end
        res << "/FirstChar #{@firstchar}\n" unless @firstchar.nil?
        res << "/LastChar #{@lastchar}\n" unless @lastchar.nil?
        res << "/Widths #{@widths} 0 R\n" unless @widths.nil?
        res << "/FontDescriptor #{@fontdescriptor} 0 R\n" unless @fontdescriptor.nil?
        res << ">>\nendobj"
      end
    end
  end
end
