module PDF
  class Writer
      # A font descriptor, needed for including additional fonts. +options+ is
      # a Hash with one of the following keys: Ascent, CapHeight, Descent,
      # Flags, ItalicAngle, StemV, AvgWidth, Leading, MaxWidth, MissingWidth,
      # StemH, XHeight, CharSet, FontFile, FontFile2, FontFile3, FontBBox, or
      # FontName.
    class FontDescriptor < PDF::Writer::WriterObject
      def initialize(parent, options = nil)
        super(parent)

        @options = options
      end

      attr_accessor :options

      def to_s
        res = "\n#{@id} 0 obj\n<< /Type /FontDescriptor\n"
        @options.each do |k, v|
          res << "/#{k} #{v}\n" if %w{Ascent CapHeight Descent Flags ItalicAngle StemV AvgWidth Leading MaxWidth MissingWidth StemH XHeight CharSet}.include?(k)
          res << "/#{k} #{v} 0 R\n" if %w{FontFile FontFile2 FontFile3}.include?(k)
          res << "/#{k} [#{v.join(' ')}]\n" if k == "FontBBox"
          res << "/#{k} /#{v}\n" if k == "FontName"
        end
        res << "\n>>\nendobj"
      end
    end
  end
end
