module PDF
  class Writer
      # an image object, will be an XObject in the document, includes description and data
    class Image < PDF::Writer::ExternalObject
      attr_accessor :label

      def initialize(parent, data, width, height, options)
        super(parent)

        @data = data

        @info = {
          'Type'        => '/XObject',
          'Subtype'     => '/Image',
          'Width'       => width,
          'Height'      => height
        }

        if options.kind_of?(Hash)
          if options['type'].nil? or options['type'] == 'jpg'
            options['channels'] = 3 if options['channels'].nil?
            @info['ColorSpace'] = '/DeviceGray' if options['channels'] == 1
            @info['ColorSpace'] ||= '/DeviceRGB'
            @info['Filter'] = '/DCTDecode'
            @info['BitsPerComponent'] = 8
          elsif options['type'] == 'png'
            @info['Filter'] = '/FlateDecode'
            @info['DecodeParms'] = "<< /Predictor 15 /Colors #{options['ncolor']} /Columns #{width} /BitsPerComponent #{options['bitsPerComponent']}>>"
            if options['pdata']
              t = " [ /Indexed /DeviceRGB #{options['pdata'].size / 3 - 1} "
              contents = PDF::Writer::Contents.new
              contents.data = options['pdata']
              t << "#{contents.id} 0 R ]"
              @info['ColorSpace'] = t
              if options['transparency']
                case options['transparency']['type']
                when 'indexed'
                  @info['Mask'] = " [ #{options['transparency']['data']} #{options['transparency']['data']}] "
                end
              end
            else
              @info['ColorSpace'] = "/#{options['color']}"
            end
          else
            @info['BitsPerComponent'] = options['bitsPerComponent']
          end
        end

        @label = options['label']

          # assign it a place in the named resource dictionary as an external
          # object, according to the label passed in with it.
        @parent.pages << self
          # also make sure that we have the right procset object for it.
        @parent.procset << 'ImageC'
      end

      def to_s
        tmp = @data.dup
        res = "\n#{@id} 0 obj\n<<"
        @info.each { |k, v| res << "\n/#{k} #{v}"}
        if (@parent.encrypted?)
          @parent.arc4.prepare(self)
          tmp = @parent.arc4.encrypt(tmp)
        end
        res << "\n/Length #{tmp.size} >>\nstream\n#{tmp}\nendstream\nendobj\n"
        res
      end
    end
  end
end
