module PDF
  class Writer
      # A page object, it also creates a contents object to hold its contents
    class Page < PDF::Writer::WriterObject
        # Create a page. The optional +relative+ is a Hash with keys :pos =>
        # :before|:after and :rpage, the page to which this new page will be
        # added relative.
      def initialize(parent, relative = nil)
        super(parent)

        @parent.current_page = self
        @owner = @parent.current_node
        @page_number = @parent.pages.size
        @contents = []

        if relative.nil?
          @parent.pages << self
        else
          relative[:page] = self
          @parent.pages.add(relative)
        end

          # make a contents object to go with this page
        @contents << PDF::Writer::Contents.new(@parent, self)
        @parent.current_contents = @contents[-1]
        match = (@parent.pages.size % 2 == 0 ? :even : :odd)
        @parent.add_loose_objects.each { |obj, target| @contents << obj if target == :all or match == target }

        @annotations = []
      end

      attr_accessor :contents
      attr_reader :page_number

      def add_annotation(a)
        @annotations << a
      end

      def to_s
        res = "\n#{@id} 0 obj\n<< /Type /Page\n/Parent #{@owner.id} 0 R"
        unless @annotations.empty?
          res << "\n/Annots ["
          @annotations.each { |e| res << " #{e.id} 0 R"}
          res << "]"
        end

        if @contents.size == 1
          res << "\n/Contents #{@contents[0].id} 0 R"
        else
          res << "\n/Contents [\n"
          @contents.each { |c| res << "#{c.id} 0 R\n" }
          res << "]"
        end
        res << "\n>>\nendobj"
      end
    end
  end
end
