$: << "lib"

require 'pp'

# This script creates the readme.pdf. This is not seriously suggested as a
# good way to create such a file, nor a great example of prose, but hopefully
# it will be useful.

begin
$TRACE = File.open("trace.out", "w")

def trace_inspect(pdf, type = "PDF", recurse = true)
  $TRACE.puts "\n#{type} [#{pdf.instance_variables.size} instance variables]"
  pdf.instance_variables.sort.each do |iv|
    next if iv =~ /@fonts/
    next if iv =~ /@font_families/
    v = pdf.instance_eval(iv)
#    trace_inspect(v, iv, false) if iv =~ /@pages/
    trace_inspect(v, iv, false) if recurse and v.class.to_s =~ /PDF::Writer/
    $TRACE.print " #{iv} "
    PP.pp(v, $TRACE)
  end
end

require 'cgi' # For CGI.escape and CGI.unescape
require 'pdf/ezwriter'
require 'progressbar'

  # Define a class extension to allow the use of a callback to get the table
  # of contents, and to put the dots in the TOC.
class Report < PDF::EZWriter
  def initialize(p, o)
    super(p, o)
    @report_contents = []
  end

  attr_accessor :report_contents

    # places a destination marker there so that it can be linked too.
  def rf(info)
    lvl = info[:p][0]
    lbl = CGI.unescape(info[:p][1.. - 1])
    num = ez_what_page_number(ez_get_current_page_number)
    @report_contents << [lbl, num, lvl]
    add_destination("toc#{@report_contents.size - 1}", 'FitH', info[:y] + info[:height])
  end

  def dots(info)
      # draw a dotted line over to the right and put on a page number
    # draw a dotted line over to the right and put on a page number
    lvl = info[:p][0]
    lbl = info[:p][1..-1]
    xpos = 520

    case lvl
    when ?1
      size = 16
      thick = 1
    when ?2
      size = 12
      thick = 0.5
    end

    save_state
    set_line_style(thick, :round, nil, [0, 10])
    line(xpos, info[:y], info[:x] + 5, info[:y])
    restore_state
    add_text(xpos + 5, info[:y], size, lbl)
  end
end

  # The author of cPDF is in NZ, so will design for A4 paper. This is the same
  # as the default, by the way.
pdf = Report.new("A4", :portrait)
pdf.ez_set_margins(50, 70, 50, 50)

  # put a line top and bottom on all the pages
all = pdf.open_object
pdf.save_state
pdf.set_stroke_color(PDF::Color.new(0, 0, 0), 1)
pdf.line(20, 40, 578, 40)
pdf.line(20, 822, 578, 822)
pdf.add_text(50, 34, 6, "http://www.sourceforge.net/projects/ruby-pdf")
pdf.restore_state
pdf.close_object
  # note that object can be told to appear on just odd or even pages by
  # changing :all to :odd or :even.
pdf.add_object(all, :all)

pdf.ez_set_dy(-100)

# mainFont = "pdf/fonts/Helvetica"
mainFont = "pdf/fonts/Times-Roman"
codeFont = "pdf/fonts/Courier"
  # select a font
pdf.select_font(mainFont)

pdf.ez_text("PDF::Writer\n", 30, { :justification => :centre })
pdf.ez_text("- Ruby PDF Creation -\n", 24, { :justification => :centre })
pdf.ez_text("Native Ruby PDF Document Creation\n", 20, { :justification => :centre })
pdf.ez_text("Developed by HaloStatue\n<c:alink:http://www.halostatue.ca/ruby/pdf>http://www.halostatue.ca/ruby/pdf</c:alink>",
            18, { :justification => :centre })
pdf.ez_text("<c:alink:http://sourceforge.net/projects/ruby-pdf>http://sourceforge.net/projects/ruby-pdf</c:alink>\nversion 0.09",
            18, { :justification => :centre })
pdf.ez_set_dy(-100)

  # modified to use the local file if it can
pdf.open_here('Fit')

def hslogo(pdf, x, y, height, left = 0, right = 0)
  pdf.save_state

  pdf.select_font("pdf/fonts/Helvetica")

  text_size = height

  text_height     = pdf.get_font_height(text_size)
  text_descender  = pdf.get_font_descender(text_size)
  halo_tw         = pdf.get_text_width(text_size, "Halo")

  pdf.set_color(PDF::Color.new(0.6, 0, 0))
  pdf.add_text(x, y - text_height - text_descender, text_size, "Halo")
  pdf.set_color(PDF::Color.new(0, 0, 0.6))
  pdf.add_text(x + halo_tw, y - text_height - text_descender, text_size, "<i>Statue</i>")

  z_factor = 0.86
  w_factor = 1.2
  s_factor = 0.1

  pdf.restore_state
  height
end

hslogo(pdf, 150, pdf.y - 100, 80, 150, 200)
pdf.select_font(mainFont)

  # load up the document content
data = IO.readlines("data.txt")

pdf.ez_start_page_numbers(500, 28, 10, nil, nil, 1)
pdf.ez_new_page

size = 12
height = pdf.get_font_height(size)
textOptions = { :justification => :full }
collecting = false

code = ""

pbar = ProgressBar.new("Data.txt", data.size)
lno = 0
data.each do |line|
  pbar.inc
  lno += 1
    # Go through each line, showing it as required, if it is surrounded by
    # '<>' then assume that it is a title.
  line.chomp!
    # comment, or new page request
  if line[0] == ?#
    case line
    when "#NP"
      pdf.ez_new_page
    when "#C"
      pdf.select_font(codeFont)
      textOptions = { :justification => :left, :left => 20, :right => 20 }
      size = 10
    when "#c"
      pdf.select_font(mainFont)
      textOptions = { :justification => :full }
      size = 12
    when "#X"
      collecting = true
    when "#x"
      collecting = false
      pdf.save_state
      begin
        eval(code)
      rescue SyntaxError, NameError => ex
        raise ex, "Error around line #{lno}:\n#{ex} [#{ex.backtrace.join("\n")}]\n"
      rescue => ex
        raise ex, "Error around line #{lno}:\n#{ex} [#{ex.backtrace.join("\n")}]\n"
      end
      pdf.restore_state
      pdf.select_font(mainFont)
      code = ""
    end
  elsif collecting
    code << "#{line}"
  elsif line[1] == ?< and line[-1] == ?> # this is a title
    case line[0]
    when ?1
      tmp = line[2..-2]
      tmp2 = "#{tmp}<C:rf:1#{CGI.escape(tmp)}>"
      pdf.ez_text(tmp2, 26, :justification => :centre)
    else
      tmp = line[2..-2]
        # Add a grey bar, highlighting the change
      tmp2 = "#{tmp}<C:rf:2#{CGI.escape(tmp)}>"
      pdf.start_transaction(:level)
      ok = false
      loop do # while not ok
        break if ok
        thisPageNum = pdf.ez_pages.size
        pdf.save_state
        pdf.set_color(PDF::Color.new(0.9, 0.9, 0.9))
        pdf.filled_rectangle(pdf.ez[:leftMargin],
                             pdf.y - pdf.get_font_height(18) + pdf.get_font_descender(18),
                             pdf.ez[:pageWidth] - pdf.ez[:leftMargin] - pdf.ez[:rightMargin],
                             pdf.get_font_height(18))
        pdf.restore_state
        pdf.ez_text(tmp2, 18, :justification => :left)
        if (pdf.ez_pages.size == thisPageNum)
          pdf.commit_transaction(:level)
          ok = true
        else
            # then we have moved onto a new page, bad bad, as the background colour will be on the old one
          pdf.rewind_transaction(:level)
          pdf.ez_new_page
        end
      end
    end
  else
    line = "\n" if line.empty?
      # Then this is just text. The ezpdf function will take care of all of
      # the wrapping etc.
    pdf.ez_text(line, size, textOptions)
  end
end

pbar.finish
pdf.ez_stop_page_numbers(true, true)

  # now add the table of contents, including internal links
pdf.ez_insert_mode(true, 1, :after)
pdf.ez_new_page
pdf.ez_text("Contents", 26, { :justification => :centre })
xpos = 520

pbar = ProgressBar.new("Contents", pdf.report_contents.size)
pdf.report_contents.each_with_index do |o, i|
  pbar.inc
  case o[2]
  when ?1
    y = pdf.ez_text("<c:ilink:toc#{i}>#{o[0]}</c:ilink><C:dots:1#{o[1]}>", 16, :aright => xpos)
  when ?2
    pdf.ez_text("<c:ilink:toc#{i}>#{o[0]}</c:ilink><C:dots:2#{o[1]}>", 12, :left => 50, :aright => xpos)
  end
end
pbar.finish

output = pdf.ez_output
File.open("readme.rb.pdf", "wb") { |f| f << output }
ensure
  $TRACE.close
end
